@@ -5,6 +5,7 @@ import mimetypes
 import os
 from typing import Tuple
 
+from google.api_core import retry
 import requests
 import requests.exceptions
 
@@ -43,6 +44,18 @@ class Client:
             labelbox.exceptions.AuthenticationError: If no `api_key`
                 is provided as an argument or via the environment
                 variable.
+
+        Logging:
+            Logging is defaulted to level WARNING. To receive more verbose 
+                output to console, update logging.level to the 
+                appropriate level.
+
+            Example:
+                #get updated on console when client is instantiated
+                import logger
+
+                logging.basicConfig(level = logging.INFO)
+                client = Client("<APIKEY>")                
         """
         if api_key is None:
             if _LABELBOX_API_KEY not in os.environ:
@@ -60,6 +73,8 @@ class Client:
             'Authorization': 'Bearer %s' % api_key
         }
 
+    @retry.Retry(predicate=retry.if_exception_type(
+        labelbox.exceptions.InternalServerError))
     def execute(self, query, params=None, timeout=10.0):
         """ Sends a request to the server for the execution of the
         given query. Checks the response for errors and wraps errors
@@ -121,12 +136,15 @@ class Client:
                 "Unknown error during Client.query(): " + str(e), e)
 
         try:
-            response = response.json()
+            r_json = response.json()
         except:
+            error_502 = '502 Bad Gateway'
+            if error_502 in response.text:
+                raise labelbox.exceptions.InternalServerError(error_502)
             raise labelbox.exceptions.LabelboxError(
                 "Failed to parse response as JSON: %s" % response.text)
 
-        errors = response.get("errors", [])
+        errors = r_json.get("errors", [])
 
         def check_errors(keywords, *path):
             """ Helper that looks for any of the given `keywords` in any of
@@ -166,16 +184,32 @@ class Client:
                 graphql_error["message"])
 
         # Check if API limit was exceeded
-        response_msg = response.get("message", "")
+        response_msg = r_json.get("message", "")
         if response_msg.startswith("You have exceeded"):
             raise labelbox.exceptions.ApiLimitError(response_msg)
 
+        prisma_error = check_errors(["INTERNAL_SERVER_ERROR"], "extensions",
+                                    "code")
+        if prisma_error:
+            raise labelbox.exceptions.InternalServerError(
+                prisma_error["message"])
+
         if len(errors) > 0:
             logger.warning("Unparsed errors on query execution: %r", errors)
             raise labelbox.exceptions.LabelboxError("Unknown error: %s" %
                                                     str(errors))
 
-        return response["data"]
+        # if we do return a proper error code, and didn't catch this above
+        # reraise
+        # this mainly catches a 401 for API access disabled for free tier
+        # TODO: need to unify API errors to handle things more uniformly
+        # in the SDK
+        if response.status_code != requests.codes.ok:
+            message = f"{response.status_code} {response.reason}"
+            cause = r_json.get('message')
+            raise labelbox.exceptions.LabelboxError(message, cause)
+
+        return r_json["data"]
 
     def upload_file(self, path: str) -> str:
         """Uploads given path to local file.
@@ -275,6 +309,7 @@ class Client:
 
     def get_project(self, project_id):
         """ Gets a single Project with the given ID.
+            >>> project = client.get_project("<project_id>")
 
         Args:
             project_id (str): Unique ID of the Project.
@@ -288,6 +323,7 @@ class Client:
 
     def get_dataset(self, dataset_id):
         """ Gets a single Dataset with the given ID.
+            >>> dataset = client.get_dataset("<dataset_id>")
 
         Args:
             dataset_id (str): Unique ID of the Dataset.
@@ -300,11 +336,17 @@ class Client:
         return self._get_single(Dataset, dataset_id)
 
     def get_user(self):
-        """ Gets the current User database object. """
+        """ Gets the current User database object. 
+            >>> user = client.get_user()
+        
+        """
         return self._get_single(User, None)
 
     def get_organization(self):
-        """ Gets the Organization DB object of the current user. """
+        """ Gets the Organization DB object of the current user. 
+            >>> organization = client.get_organization()
+        
+        """
         return self._get_single(Organization, None)
 
     def _get_all(self, db_object_type, where):
@@ -327,6 +369,7 @@ class Client:
 
     def get_projects(self, where=None):
         """ Fetches all the projects the user has access to.
+            >>> projects = client.get_projects(where=(Project.name == "<project_name>") & (Project.description == "<project_description>"))
 
         Args:
             where (Comparison, LogicalOperation or None): The `where` clause
@@ -337,7 +380,8 @@ class Client:
         return self._get_all(Project, where)
 
     def get_datasets(self, where=None):
-        """ Fetches all the datasets the user has access to.
+        """ Fetches one or more datasets.
+            >>> datasets = client.get_datasets(where=(Dataset.name == "<dataset_name>") & (Dataset.description == "<dataset_description"))
 
         Args:
             where (Comparison, LogicalOperation or None): The `where` clause
@@ -349,6 +393,7 @@ class Client:
 
     def get_labeling_frontends(self, where=None):
         """ Fetches all the labeling frontends.
+            >>> frontend = client.get_labeling_frontends(where=LabelingFrontend.name == "Editor")
 
         Args:
             where (Comparison, LogicalOperation or None): The `where` clause
@@ -388,9 +433,8 @@ class Client:
     def create_dataset(self, **kwargs):
         """ Creates a Dataset object on the server. Attribute values are
             passed as keyword arguments:
-                >>> project = client.get_project("uid_of_my_project")
-                >>> dataset = client.create_dataset(name="MyDataset",
-                >>>                                 projects=project)
+                >>> project = client.get_project("<project_uid>")
+                >>> dataset = client.create_dataset(name="<dataset_name>", projects=project)
 
         Kwargs:
             Keyword arguments with new Dataset attribute values.
@@ -407,7 +451,7 @@ class Client:
     def create_project(self, **kwargs):
         """ Creates a Project object on the server. Attribute values are
             passed as keyword arguments:
-                >>> project = client.create_project(name="MyProject")
+                >>> project = client.create_project(name="<project_name>", description="<project_description>")
 
         Kwargs:
             Keyword arguments with new Project attribute values.
