@@ -37,7 +37,9 @@ class DataRow(DbObject, Updateable, BulkDeletable):
 
     def create_metadata(self, meta_type, meta_value):
         """ Creates an asset metadata for this DataRow.
-
+        
+        >>> datarow.create_metadata("TEXT", "This is a text message")
+        
         Args:
             meta_type (str): Asset metadata type, must be one of:
                 VIDEO, IMAGE, TEXT.
@@ -49,7 +51,7 @@ class DataRow(DbObject, Updateable, BulkDeletable):
         meta_value_param = "metaValue"
         data_row_id_param = "dataRowId"
         query_str = """mutation CreateAssetMetadataPyApi(
-            $%s: MetadataType!, $%s: String!, $%s: ID!) {
+            $%s: AttachmentType!, $%s: String!, $%s: ID!) {
             createAssetMetadata(data: {
                 metaType: $%s metaValue: $%s dataRowId: $%s}) {%s}} """ % (
             meta_type_param, meta_value_param, data_row_id_param,
