@@ -15,6 +15,12 @@ from labelbox.orm.db_object import DbObject, Updateable, Deletable
 from labelbox.orm.model import Entity, Field, Relationship
 from labelbox.pagination import PaginatedCollection
 
+try:
+    datetime.fromisoformat  # type: ignore[attr-defined]
+except AttributeError:
+    from backports.datetime_fromisoformat import MonkeyPatch
+    MonkeyPatch.patch_fromisoformat()
+
 logger = logging.getLogger(__name__)
 
 
@@ -46,9 +52,11 @@ class Project(DbObject, Updateable, Deletable):
     active_prediction_model = Relationship.ToOne("PredictionModel", False,
                                                  "active_prediction_model")
     predictions = Relationship.ToMany("Prediction", False)
+    ontology = Relationship.ToOne("Ontology", True)
 
     def create_label(self, **kwargs):
-        """ Creates a label on this Project.
+        """ Creates a label on a Legacy Editor project. Not
+            supported in the new Editor.
 
         Kwargs:
             Label attributes. At the minimum the label `DataRow`.
@@ -161,8 +169,9 @@ class Project(DbObject, Updateable, Deletable):
 
         def create_labeler_performance(client, result):
             result["user"] = Entity.User(client, result["user"])
-            result["lastActivityTime"] = datetime.fromtimestamp(
-                result["lastActivityTime"] / 1000, timezone.utc)
+            # python isoformat doesn't accept Z as utc timezone
+            result["lastActivityTime"] = datetime.fromisoformat(
+                result["lastActivityTime"].replace('Z', '+00:00'))
             return LabelerPerformance(
                 **
                 {utils.snake_case(key): value for key, value in result.items()})
@@ -298,7 +307,8 @@ class Project(DbObject, Updateable, Deletable):
         return res["extendReservations"]
 
     def create_prediction_model(self, name, version):
-        """ Creates a PredictionModel connected to this Project.
+        """ Creates a PredictionModel connected to a Legacy Editor
+            Project.
         Args:
             name (str): The new PredictionModel's name.
             version (int): The new PredictionModel's version.
@@ -314,7 +324,8 @@ class Project(DbObject, Updateable, Deletable):
         return model
 
     def create_prediction(self, label, data_row, prediction_model=None):
-        """ Creates a Prediction within this Project.
+        """ Creates a Prediction within a Legacy Editor Project.
+        
         Args:
             label (str): The `label` field of the new Prediction.
             data_row (DataRow): The DataRow for which the Prediction is created.
@@ -361,7 +372,7 @@ class Project(DbObject, Updateable, Deletable):
         name: str,
         annotations: Union[str, Union[str, Path], Iterable[dict]],
     ) -> 'BulkImportRequest':  # type: ignore
-        """ Uploads annotations to a project.
+        """ Uploads annotations to a new Editor project.
 
         Args:
             name: name of the BulkImportRequest job
@@ -410,13 +421,16 @@ class Project(DbObject, Updateable, Deletable):
                     file=path,
                     validate_file=True,
                 )
-        else:
+        elif isinstance(annotations, Iterable):
             return BulkImportRequest.create_from_objects(
                 client=self.client,
                 project_id=self.uid,
                 name=name,
                 predictions=annotations,  # type: ignore
             )
+        else:
+            raise ValueError(
+                f'Invalid annotations given of type: {type(annotations)}')
 
 
 class LabelingParameterOverride(DbObject):
